import sys
import subprocess
import argparse
from time import perf_counter
from pathlib import Path

#===================================================================================================
# region: FUNCTIONS
#===================================================================================================
def parse_args():
    # create parser object
    parser = argparse.ArgumentParser(description = "Send all the variants to be evaluated")
 
    # defining arguments for parser object
    parser.add_argument('-v', '--var', type = str, 
                        metavar = 'DIR', 
                        help = 'The directory path containing all the design variants')
    
    parser.add_argument('-r', '--res', type = str, 
                        metavar = 'DIR', 
                        help = 'The directory path for the results')
    
    parser.add_argument('-e', '--epw', type = str,
                        metavar = 'FILE',
                        help = 'The file path of the weather file')
    
    parser.add_argument('-d', '--ddy', type = str,
                        metavar = 'FILE',
                        help = 'The file path of the ddy design day file')
    
    parser.add_argument('-m', '--measure', type = str, default=None,
                        metavar = 'FILE',
                        help = 'The file path of the json measures file that specify which measures to apply to the model')
    
    parser.add_argument('-p', '--process', action = 'store_true', default=False,
                        help = 'turn it on if piping in json filepath')
    
    # parse the arguments from standard input
    args = parser.parse_args()
    return args

def batch_eval_variants(var_dir: str, res_dir: str, epw_path: str, ddy_path: str, measure_path: str):
    '''
    Execute a parameteric model and generate a variant.

    Parameters
    ----------
    var_dir: str
        The directory path of the design variants.

    res_dir : str
        The path of the directory.
    
    epw_path : str
        The file path of the weather file.

    ddy_path : str
        The file path of the ddy design day file.

    measure_path : str
        The file path of the measures that will be applied to the model.
    
    '''
    filesx = Path(var_dir).glob('*.ifc')
    filesx = sorted(filesx)
    res_dir_pobj = Path(res_dir)
    call_list1 = ['ifcarch2osmod', '-i', '', '-o', '' ]
    call_list2 = ['add_sch2osmod', '-p', '-b', 'Small Office', '-c', '1A' ]
    call_list3 = ['execute_osmod', '-p', '-e', epw_path, '-d', ddy_path, '-m', measure_path, '-out', '']
    call_list4 = ['epsql2csv', '-s', '', '-r', '']

    t1 = perf_counter()
    for filex in filesx:
        t2 = perf_counter()
        filename = filex.stem
        this_res_dir = res_dir_pobj.joinpath(filename)
        this_res_dir.mkdir(parents=True, exist_ok=True)
        path_str = str(filex)
        call_list1[2] = path_str
        osm_path = this_res_dir.joinpath(f"{filename}.osm")
        call_list1[4] = str(osm_path)
        call_list3[9] = str(this_res_dir)
        
        sql_path = this_res_dir.joinpath(f"{filename}_wrkflw", 'run', 'eplusout.sql')
        csv_dir = this_res_dir.joinpath('csv')
        csv_dir.mkdir(parents=True, exist_ok=True)
        call_list4[2] = str(sql_path)
        call_list4[4] = str(csv_dir)
        print(f"executing openstudio model ... {filename}")
        try:
            process1 = subprocess.Popen(call_list1, stdout=subprocess.PIPE)
            process2 = subprocess.Popen(call_list2, stdin=process1.stdout, stdout=subprocess.PIPE)
            process3 = subprocess.Popen(call_list3, stdin=process2.stdout, stdout=subprocess.PIPE)
            output = process3.communicate()[0]
            print(output.decode())
        except subprocess.CalledProcessError as e:
            print("An error occurred:", e)

        res = subprocess.run(call_list4, capture_output=True, text=True)
        print(res.stdout)
        print(res.stderr)
        t3 = perf_counter()
        t32 = round((t3-t2)/60, 1)
        print(f"{t32} mins")

    t4 = perf_counter()
    t41 = round((t4 - t1)/60, 1)
    print(f"{t41} mins")
    
def main():
    args = parse_args()
    pipe_input = args.process
    if pipe_input == False:
        var_dir = args.var
    else:
        lines = list(sys.stdin)
        var_dir = lines[0].strip()

    var_dir = str(Path(var_dir).resolve())
    res_dir = args.res
    res_dir = str(Path(res_dir).resolve())
    epw_path = str(Path(args.epw).resolve())
    ddy_path = str(Path(args.ddy).resolve())
    mea_path = str(Path(args.measure).resolve())
    batch_eval_variants(var_dir, res_dir, epw_path, ddy_path, mea_path)
    # make sure this output can be piped into another command on the cmd
    print(res_dir)
    sys.stdout.flush()
#===================================================================================================
# endregion: FUNCTIONS
#===================================================================================================
#===================================================================================================
# region: Main
#===================================================================================================
if __name__=='__main__':
    main()
#===================================================================================================
# endregion: Main
#===================================================================================================


import sys
import json
import argparse
from pathlib import Path

import ifcopenshell
import ifcopenshell.geom

import ifc_utils
#===================================================================================================
# region: FUNCTIONS
#===================================================================================================
def parse_args():
    # create parser object
    parser = argparse.ArgumentParser(description = "Parameterize IFC model")
 
    # defining arguments for parser object
    parser.add_argument('-i', '--ifc', type = str, 
                        metavar = 'FILE', 
                        help = 'The file path of the IFC to convert')
    
    parser.add_argument('-r', '--res', type = str, 
                        metavar = 'FILE', 
                        help = 'The file path of the resultant json file')
    
    parser.add_argument('-p', '--process', action = 'store_true', default=False,
                        help = 'turn it on if piping in ifc filepath')
    
    # parse the arguments from standard input
    args = parser.parse_args()
    return args

def pmtrz_wwr_constr(ifc_path: str, res_path: str):
    '''
    Parameterize an ifc file.

    Parameters
    ----------
    ifc_path : str
        The file path of ifc.

    res_path : str
        The file path of the resultant json file.
    
    '''
    ifcmodel = ifcopenshell.open(ifc_path)
    bldg_dict = ifc_utils.ifcopenshell_utils.get_ifc_building_info(ifcmodel, envlp_pset_name='Pset_OsmodThermalResistance')
    ifc_bldgs = list(bldg_dict.values())
    nbldgs = len(ifc_bldgs)
    if nbldgs == 1:
        pmtrc_mod = {}
        pmtrc_mod['exe_script'] = 'exe_wwr_constr'
        pmtrs = {}
        pmtrs['wall_thermal_resistance'] = {'range': [0.5, 3]}
        pmtrs['roof_thermal_resistance'] = {'range': [3, 6]}
        pmtrs['floor_thermal_resistance'] = {'range': [3, 6]}
        pmtrs['glazing_uvalue'] = {'range': [0.5, 3]}
        pmtrs['north_wwr'] = {'range': [0.1, 0.4]}
        pmtrs['south_wwr'] = {'range': [0.1, 0.4]}
        pmtrs['east_wwr'] = {'range': [0.1, 0.4]}
        pmtrs['west_wwr'] = {'range': [0.1, 0.4]}
        pmtrc_mod['parameters'] = pmtrs

        # pretty json
        pretty_json_data = json.dumps(pmtrc_mod, indent=4)
        with open(res_path, 'w') as f:
            f.write(pretty_json_data)
    else:
        raise Exception("Unexpected number of buildings", nbldgs, "only 1 building allowed")

def main():
    args = parse_args()
    pipe_input = args.process
    if pipe_input == False:
        ifc_path = args.ifc
    else:
        lines = list(sys.stdin)
        ifc_path = lines[0].strip()

    res_path = args.res
    res_path = str(Path(res_path).resolve())
    ifc_path = str(Path(ifc_path).resolve())
    pmtrz_wwr_constr(ifc_path, res_path)
    # make sure this output can be piped into another command on the cmd
    print(res_path)
    sys.stdout.flush()
#===================================================================================================
# endregion: FUNCTIONS
#===================================================================================================
#===================================================================================================
# region: Main
#===================================================================================================
if __name__=='__main__':
    main()
#===================================================================================================
# endregion: Main
#===================================================================================================


import sys
import json
import argparse
from pathlib import Path

from pymoo.core.problem import Problem
from pymoo.operators.sampling.lhs import LHS
import jsonschema

from . import settings
#===================================================================================================
# region: FUNCTIONS
#===================================================================================================
def parse_args():
    # create parser object
    parser = argparse.ArgumentParser(description = "Generate design variants using sampling algorithm")
 
    # defining arguments for parser object
    parser.add_argument('-n', '--nsamples', type = int, 
                        metavar = 'NSAMPLES', 
                        help = 'The number of variants to generate')
    
    parser.add_argument('-j', '--json', type = str, 
                        metavar = 'FILE', 
                        help = 'The file path of the json parametric file')
    
    parser.add_argument('-r', '--res', type = str, 
                        metavar = 'FILE', default= None,
                        help = 'The file path for the result')
    
    parser.add_argument('-p', '--process', action = 'store_true', default=False,
                        help = 'turn it on if piping in json filepath')
    
    # parse the arguments from standard input
    args = parser.parse_args()
    return args

def sample_pmtrs(nsamples: int, pmtrc_path: str, res_path: str):
    '''
    Generate parameters for the model.

    Parameters
    ----------
    nsamples: int
        number of samples to generate

    pmtrc_path: str
        The file path of the json parametric model.

    ifc_path : str
        The file path of ifc.

    res_path : str
        The path of the result file.
    
    var_dir : str
        The directory to store the generated variants.

    '''
    with open(pmtrc_path) as pmtrc_file:
        pmtrc_mod = json.load(pmtrc_file)
    
    # validate the file and make sure it is compliant
    json_data_dir = settings.JSON_DATA_DIR
    pmtrc_mod_schema_path = Path(json_data_dir).joinpath('pmtrc_mod_schema.json')
    with open(pmtrc_mod_schema_path) as schema_file:
        pmtrc_schema = json.load(schema_file)

    try:
        is_mod_valid = jsonschema.validate(instance=pmtrc_mod, schema=pmtrc_schema)
    except jsonschema.ValidationError as e:
        print(e.schema.get("error_msg", e.message))
        return False 
    
    pmtrs_dict = pmtrc_mod['parameters']
    pmtrs_vals = pmtrs_dict.values()
    npmtrs = len(pmtrs_vals)
    problem = Problem(n_var=npmtrs, xl=0, xu=1)
    sampling = LHS()
    pmtrs_nrmlz = sampling(problem, nsamples).get("X")
    pmtrs_nrmlz = pmtrs_nrmlz.tolist()
    pmtrc_mod['parameter_normalized_values'] = pmtrs_nrmlz

    pretty_json_data = json.dumps(pmtrc_mod, indent=4)
    with open(res_path, 'w') as f:
        f.write(pretty_json_data)
    return True

def main():
    args = parse_args()
    pipe_input = args.process
    if pipe_input == False:
        pmtrc_path = args.json
    else:
        lines = list(sys.stdin)
        pmtrc_path = lines[0].strip()

    nsamples = args.nsamples
    res_path = args.res
    if res_path == None:
        res_path = pmtrc_path
        
    res_path = str(Path(res_path).resolve())

    is_executed = sample_pmtrs(nsamples, pmtrc_path, res_path)
    if is_executed:
        # make sure this output can be piped into another command on the cmd
        print(res_path)
        sys.stdout.flush()
#===================================================================================================
# endregion: FUNCTIONS
#===================================================================================================
#===================================================================================================
# region: Main
#===================================================================================================
if __name__=='__main__':
    main()
#===================================================================================================
# endregion: Main
#===================================================================================================